#!/usr/bin/python
#-----------------------------------------------------------------------
# File    : hubness.py
# Contents: execute hubness program for several parameter settings
# Author  : Christian Borgelt
# History : 2012.01.13 file created
#           2012.01.18 external script for repeated call removed
#           2012.02.07 seed value sequence improved (size dependent)
#-----------------------------------------------------------------------
from sys        import argv
from math       import sqrt
from os         import devnull
from subprocess import check_output
from id2par     import run

#-----------------------------------------------------------------------

def hubs (s='c', n=1000, d=2, k=5, g=0, r=10):
    fname = 's' +s +'_n' +str(n) +'_d' +str(d) \
                   +'_k' +str(k) +'_g' +str(g) +'.txt'
    prog  = [ 'hubness', '-s'+str(s), '-n'+str(n), '-d'+str(d),
                         '-k'+str(k), '-g'+str(g), '-' ]
    with open(fname, 'w') as out:
        sums = sqrs = ()
        seed = int(1 +(n/10000 if n >= 10000 else n/1000))
        for i in xrange(r):
            with open(devnull, 'w') as null:
                res = check_output(prog +['-S'+str(seed)], stderr=null)
            out.write(res)
            res  = [ float(x) for x in res.split() ]
            if sums == ():
                sums = res
                sqrs = [ y*y for y in res ]
            else:
                sums = [ x+y   for x,y in zip(sums,res) ]
                sqrs = [ x+y*y for x,y in zip(sqrs,res) ]
            seed += 10
        a = 1.0/ r    if r > 0 else 1
        z = 1.0/(r-1) if r > 1 else 1
        sqrs = [ sqrt((y -x*x*a)*z) for x,y in zip(sums,sqrs) ]
        sums = [ x*a for x in sums ]
        out.write('----------------\n')
        out.write(str(sums[0]))
        for x in sums[1:]: out.write(' ' +str(x))
        out.write('\n')
        out.write(str(sqrs[0]))
        for x in sqrs[1:]: out.write(' ' +str(x))
        out.write('\n')

#-----------------------------------------------------------------------

if __name__ == '__main__':
    # parameters for (hyper-)cubes, (hyper)balls etc.
    # grids = [ [ ( 's', [ 'c', 'b', 'n', 'j' ] ),
    #             ( 'n', [ 10000, 20000 ] ),
    #             ( 'd', [ 2, 3, 5, 10, 15, 20, 30, 40, 50, 60, 80, 100 ] ),
    #             ( 'k', [ 5, 10, 20 ] ),
    #             ( 'g', [ 0 ] ),
    #             ( 'r', [ 100 ] ) ] ]

    # parameters for (hyper-)surfaces
    # grids = [ [ ( 's', [ 's' ] ),
    #             ( 'n', [ 10000, 20000 ] ),
    #             ( 'd', [ 3, 4, 6, 11, 16, 21, 31, 41, 51, 61, 81, 101 ] ),
    #             ( 'k', [ 5, 10, 20 ] ),
    #             ( 'g', [ 0 ] ),
    #             ( 'r', [ 100 ] ) ] ]

    # parameters for grids of cubes
    # grids = [ [ ( 's', [ 'c' ] ),
    #             ( 'n', [ 10000, 20000 ] ),
    #             ( 'd', [ 2, 3 ] ),
    #             ( 'k', [ 5, 8 ] ),
    #             ( 'g', [ 2.5, 4.5, 6.5, 8.5, 10.5,
    #                      8.6, 8.7, 8.8, 8.9,  0.0 ] ),
    #             ( 'r', [ 100 ] ) ] ]

    # all parameters (with minor redundancies)
    grids = [ [ ( 's', [ 'c', 'b', 'n', 'j' ] ),
                ( 'n', [ 10000, 20000 ] ),
                ( 'd', [ 2, 3, 5, 10, 15, 20, 30, 40, 50, 60, 80, 100 ] ),
                ( 'k', [ 10 ] ),
                ( 'g', [ 0 ] ),
                ( 'r', [ 200 ] ) ],
              [ ( 's', [ 's' ] ),
                ( 'n', [ 10000, 20000 ] ),
                ( 'd', [ 3, 4, 6, 11, 16, 21, 31, 41, 51, 61, 81, 101 ] ),
                ( 'k', [ 10 ] ),
                ( 'g', [ 0 ] ),
                ( 'r', [ 200 ] ) ],
              [ ( 's', [ 'c' ] ),
                ( 'n', [ 10000, 20000 ] ),
                ( 'd', [ 3 ] ),
                ( 'k', [ 5, 8 ] ),
                ( 'g', [ 2.5, 4.5, 6.5, 8.5, 10.5,
                         8.6, 8.7, 8.8, 8.9,  0.0 ] ),
                ( 'r', [ 200 ] ) ] ]

    run(hubs, grids, int(argv[1])-1 if len(argv) > 1 else -1)
